==> SD35B_0.02.1_src.tar.gz: Sd35b/decoder/AliceCG_PMS.cpp <==
/*
 * AliceCG_PMS.cpp
 *		 - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * pms256.c@xsystem35  PMS256 WJ
 * pms64k.c@xsystem35  PMS 6F WJ
 *
 * Copyright (C) 1997-1998 Masaki Chikama (Wren) <chikama@kasumi.ipl.mech.nagoya-u.ac.jp>
 *               1998-                           <masaki-c@is.aist-nara.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * pms256.c
 *
 *          0.00    97/11/06 
 *          0.01    97/11/13 wb_̕Ԃ̕ύX
 *          0.01-01 97/11/27 Lq̕ύX
 * @version 0.02    98/02/26 fobO
 *
 * pms64k.c
 *          0.00 97/11/06 
 *          0.01 97/11/13 wb_̕Ԃ̕ύX
 *          0.02 98/02/26 fobO
 * @version 0.03 98/07/27 Ձ`BF̓WJ܂ĂB
 *
 *	99/05/03 Atsushi Mizuguchi C++ogݍ 
 *
 */

#include "stdafx.h"
#include "AliceCG_PMS.h"
#include "EALSD.h"
#include "Endian.h"

using namespace endian;
using namespace ALSD;
using namespace ALSD_ALCG;
/**********************************************************************
	wb_擾
**********************************************************************/
void CAliceCG_PMS::GetHeader( void )
{
	// Magic number
	if( m_lpMem[0] != 'P' || m_lpMem[1] != 'M' ){
		throw EALSD_Msg( "error:PMS Magic number(%C%C)", m_lpMem[0], m_lpMem[1] );
	}
	// GetHeader
	m_Header.Version= GetLE_WD( m_lpMem + 2 );
	m_Header.Size	= GetLE_WD( m_lpMem + 4 );
	m_Header.Bpp	= m_lpMem[6];
	m_Header.BppS	= m_lpMem[7];
	m_Header.Sf		= m_lpMem[8];
	m_Header.Bf		= GetLE_WD( m_lpMem + 10 );
					  GetLE_DW( m_lpMem + 12 ); // Dummy
	m_Header.X		= GetLE_DW( m_lpMem + 16 );
	m_Header.Y		= GetLE_DW( m_lpMem + 20 );
	m_Header.Width	= GetLE_DW( m_lpMem + 24 );
	m_Header.Height	= GetLE_DW( m_lpMem + 28 );
	m_Header.DataPtr= GetLE_DW( m_lpMem + 32 );
	m_Header.AlphaPtr=GetLE_DW( m_lpMem + 36 );
	m_Header.Cp		= GetLE_DW( m_lpMem + 40 );
}

/**********************************************************************
	pbg擾
**********************************************************************/
void CAliceCG_PMS::GetPalette( void )
{
	for( int i = 0, j = m_Header.AlphaPtr; i < 256; i++, j+= 3){
		m_Palette.entry[i].alpha	= 0;
		m_Palette.entry[i].red		= m_lpMem[j + 0];
		m_Palette.entry[i].green	= m_lpMem[j + 1];
		m_Palette.entry[i].blue		= m_lpMem[j + 2]; 
	}	
}

/**********************************************************************
	TCY̎擾
**********************************************************************/
void CAliceCG_PMS::GetCGSizeInfo( stCGSize *_lpCGSize )
{
	_lpCGSize->x = m_Header.X;
	_lpCGSize->y = m_Header.Y;
	_lpCGSize->Width = m_Header.Width;
	_lpCGSize->Height= m_Header.Height;
}


/**********************************************************************
	WJ
**********************************************************************/
void CAliceCG_PMS::Expand64K(	unsigned short *_lpPic,
								unsigned char *_lpAlpha )
{
	long c0, c1, pc0, pc1;
	long x, y, i, l, loc;
	long lpitch = m_Header.Width;
	const unsigned char *b = m_lpMem;
	unsigned short *pic = _lpPic;

	b += m_Header.DataPtr;
	
	for( y = 0; y < m_Header.Height; y++){
		for( x = 0; x < m_Header.Width; ){
			loc = y * lpitch + x;
			c0 = *b++;
			if( c0 <= 0xf7 ){
				c1 = *b++; x++;
				*(pic + loc) = c0 | (c1 << 8);
			}else if ( c0 == 0xff ){
				l = (*b) + 2; b++;
				x += l;
				for (i = 0; i < l; i++) {
					*(pic + loc + i) = *(pic + loc + i - lpitch);
				}
			}else if( c0 == 0xfe ){
				l = (*b) + 2; b++;
				x += l;
				for( i = 0; i < l; i++){
					*(pic + loc + i) = *(pic + loc + i - lpitch * 2);
				}
			}else if( c0 == 0xfd ){
				l = (*b) + 3; b++;
				x += l;
				c0 = *b++;
				c1 = *b++;
				pc0 = c0 | (c1 << 8);
				for( i = 0; i < l; i++){
					*(pic + loc + i) = pc0;
				}
			}else if( c0 == 0xfc ){
				l = ((*b) + 2) * 2; b++;
				 x += l;
				c0 = *b++; c1 = *b++; pc0 = c0 | (c1 << 8);
				c0 = *b++; c1 = *b++; pc1 = c0 | (c1 << 8);
				for( i = 0; i < l; i+=2 ) {
					*(pic + loc + i    ) = pc0;
					*(pic + loc + i + 1) = pc1;
				}
			}else if( c0 == 0xfb ){
				x++;
				*(pic + loc) = *(pic + loc - lpitch - 1);
			}else if( c0 == 0xfa ){
				x++;
				*(pic + loc) = *(pic + loc - lpitch + 1);
			}else if( c0 == 0xf9 ){
				l = (*b) + 1; b++;
				x += l;
				c0 = *b++;
				c1 = *b++;
				pc0 = ((c0 & 0xe0) << 8) + ((c0 & 0x18) << 6) + ((c0 & 0x07) << 2);
				pc1 = ((c1 & 0xc0) << 5) + ((c1 & 0x3c) << 3) + (c1 & 0x03);
				*(pic + loc) = pc0 + pc1;
				for( i = 1; i < l; i++){
					c1 = *b++;
					pc1 = ((c1 & 0xc0) << 5) + ((c1 & 0x3c) << 3) + (c1 & 0x03);
					*(pic + loc + i) = pc0 | pc1;
				}
			}else{
				c0 = *b++;
				c1 = *b++;
				x++;
				*(pic + loc) = c0 | (c1 << 8);
			}
		}
	}
	// ẽ[h
	if( m_lpShadowPMS ){
		m_lpShadowPMS->Expand256( _lpAlpha, -1 );
	}else if( m_Header.AlphaPtr ) {
		Expand256( _lpAlpha, m_Header.AlphaPtr );
	}

}

/**********************************************************************
	pbg̎擾
**********************************************************************/
void CAliceCG_PMS::ExpandPal( stALCG_Palette *_lpPal )
{
	*_lpPal = m_Palette;
}

/**********************************************************************
	256F
**********************************************************************/
void CAliceCG_PMS::Expand256( unsigned char *_lpPic, long _Ptr)
{
	int c0, c1;
	int x, y, loc, l, i;
	int lpitch = m_Header.Width;
	const unsigned char *b = m_lpMem;
	unsigned char *pic = _lpPic;
	
	if( _Ptr == -1 ){
		b += m_Header.DataPtr;
	}else{
		b += _Ptr;
	}

	for( y = 0; y < m_Header.Height; y++){
		for( x = 0; x < m_Header.Width; ){
			loc = y * lpitch + x;
			c0 = *b++;
			if( c0 <= 0xf7 ){
				*(pic + loc) = c0;
				x++;
			}else if( c0 == 0xff ){
				l = (*b) + 3; b++;
				x += l; 
				memcpy( pic + loc, pic + loc - lpitch, l);
			}else if( c0 == 0xfe ){
				l = (*b) + 3; b++;
				x += l; 
				memcpy( pic + loc, pic + loc - lpitch * 2, l);
			}else if( c0 == 0xfd ){
				l = (*b) + 4; b++;
				x += l;
				c0 = *b++;
				memset( pic + loc, c0, l);
			}else if( c0 == 0xfc ){
				l = ( (*b) + 3 )  * 2; b++;
				x += l;
				c0 = *b++;
				c1 = *b++;
				for( i = 0; i < l; i+=2){
					*(pic + loc + i    ) = c0;
					*(pic + loc + i + 1) = c1;
				}
			}else{
				*(pic + loc) = *b++;
				x++;
			}
		}
	}
}

==> SD35B_0.02.2_src.tar.gz: Sd35b/decoder/AliceCG_PMS.cpp <==
/*
 * AliceCG_PMS.cpp
 *		 - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * pms256.c@xsystem35  PMS256 WJ
 * pms64k.c@xsystem35  PMS 6F WJ
 *
 * Copyright (C) 1997-1998 Masaki Chikama (Wren) <chikama@kasumi.ipl.mech.nagoya-u.ac.jp>
 *               1998-                           <masaki-c@is.aist-nara.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * pms256.c
 *
 *          0.00    97/11/06 
 *          0.01    97/11/13 wb_̕Ԃ̕ύX
 *          0.01-01 97/11/27 Lq̕ύX
 * @version 0.02    98/02/26 fobO
 *
 * pms64k.c
 *          0.00 97/11/06 
 *          0.01 97/11/13 wb_̕Ԃ̕ύX
 *          0.02 98/02/26 fobO
 * @version 0.03 98/07/27 Ձ`BF̓WJ܂ĂB
 *
 *	99/05/03 Atsushi Mizuguchi C++ogݍ 
 *
 */

#include "stdafx.h"
#include "AliceCG_PMS.h"
#include "EALSD.h"
#include "Endian.h"

using namespace endian;
using namespace ALSD;
using namespace ALSD_ALCG;
/**********************************************************************
	wb_擾
**********************************************************************/
void CAliceCG_PMS::GetHeader( void )
{
	// Magic number
	if( m_lpMem[0] != 'P' || m_lpMem[1] != 'M' ){
		throw EALSD_Msg( "error:PMS Magic number(%C%C)", m_lpMem[0], m_lpMem[1] );
	}
	// GetHeader
	m_Header.Version= GetLE_WD( m_lpMem + 2 );
	m_Header.Size	= GetLE_WD( m_lpMem + 4 );
	m_Header.Bpp	= m_lpMem[6];
	m_Header.BppS	= m_lpMem[7];
	m_Header.Sf		= m_lpMem[8];
	m_Header.Bf		= GetLE_WD( m_lpMem + 10 );
					  GetLE_DW( m_lpMem + 12 ); // Dummy
	m_Header.X		= GetLE_DW( m_lpMem + 16 );
	m_Header.Y		= GetLE_DW( m_lpMem + 20 );
	m_Header.Width	= GetLE_DW( m_lpMem + 24 );
	m_Header.Height	= GetLE_DW( m_lpMem + 28 );
	m_Header.DataPtr= GetLE_DW( m_lpMem + 32 );
	m_Header.AlphaPtr=GetLE_DW( m_lpMem + 36 );
	m_Header.Cp		= GetLE_DW( m_lpMem + 40 );
}

/**********************************************************************
	pbg擾
**********************************************************************/
void CAliceCG_PMS::GetPalette( void )
{
	for( int i = 0, j = m_Header.AlphaPtr; i < 256; i++, j+= 3){
		m_Palette.entry[i].alpha	= 0;
		m_Palette.entry[i].red		= m_lpMem[j + 0];
		m_Palette.entry[i].green	= m_lpMem[j + 1];
		m_Palette.entry[i].blue		= m_lpMem[j + 2]; 
	}	
}

/**********************************************************************
	TCY̎擾
**********************************************************************/
void CAliceCG_PMS::GetCGSizeInfo( stCGSize *_lpCGSize )
{
	_lpCGSize->x = m_Header.X;
	_lpCGSize->y = m_Header.Y;
	_lpCGSize->Width = m_Header.Width;
	_lpCGSize->Height= m_Header.Height;
}


/**********************************************************************
	WJ
**********************************************************************/
void CAliceCG_PMS::Expand64K(	unsigned short *_lpPic,
								unsigned char *_lpAlpha )
{
	long c0, c1, pc0, pc1;
	long x, y, i, l, loc;
	long lpitch = m_Header.Width;
	const unsigned char *b = m_lpMem;
	unsigned short *pic = _lpPic;

	b += m_Header.DataPtr;
	
	for( y = 0; y < m_Header.Height; y++){
		for( x = 0; x < m_Header.Width; ){
			loc = y * lpitch + x;
			c0 = *b++;
			if( c0 <= 0xf7 ){
				c1 = *b++; x++;
				*(pic + loc) = c0 | (c1 << 8);
			}else if ( c0 == 0xff ){
				l = (*b) + 2; b++;
				x += l;
				for (i = 0; i < l; i++) {
					*(pic + loc + i) = *(pic + loc + i - lpitch);
				}
			}else if( c0 == 0xfe ){
				l = (*b) + 2; b++;
				x += l;
				for( i = 0; i < l; i++){
					*(pic + loc + i) = *(pic + loc + i - lpitch * 2);
				}
			}else if( c0 == 0xfd ){
				l = (*b) + 3; b++;
				x += l;
				c0 = *b++;
				c1 = *b++;
				pc0 = c0 | (c1 << 8);
				for( i = 0; i < l; i++){
					*(pic + loc + i) = pc0;
				}
			}else if( c0 == 0xfc ){
				l = ((*b) + 2) * 2; b++;
				 x += l;
				c0 = *b++; c1 = *b++; pc0 = c0 | (c1 << 8);
				c0 = *b++; c1 = *b++; pc1 = c0 | (c1 << 8);
				for( i = 0; i < l; i+=2 ) {
					*(pic + loc + i    ) = pc0;
					*(pic + loc + i + 1) = pc1;
				}
			}else if( c0 == 0xfb ){
				x++;
				*(pic + loc) = *(pic + loc - lpitch - 1);
			}else if( c0 == 0xfa ){
				x++;
				*(pic + loc) = *(pic + loc - lpitch + 1);
			}else if( c0 == 0xf9 ){
				l = (*b) + 1; b++;
				x += l;
				c0 = *b++;
				c1 = *b++;
				pc0 = ((c0 & 0xe0) << 8) + ((c0 & 0x18) << 6) + ((c0 & 0x07) << 2);
				pc1 = ((c1 & 0xc0) << 5) + ((c1 & 0x3c) << 3) + (c1 & 0x03);
				*(pic + loc) = pc0 + pc1;
				for( i = 1; i < l; i++){
					c1 = *b++;
					pc1 = ((c1 & 0xc0) << 5) + ((c1 & 0x3c) << 3) + (c1 & 0x03);
					*(pic + loc + i) = pc0 | pc1;
				}
			}else{
				c0 = *b++;
				c1 = *b++;
				x++;
				*(pic + loc) = c0 | (c1 << 8);
			}
		}
	}
	// ẽ[h
	if( m_lpShadowPMS ){
		m_lpShadowPMS->Expand256( _lpAlpha, -1 );
	}else if( m_Header.AlphaPtr ) {
		Expand256( _lpAlpha, m_Header.AlphaPtr );
	}

}

/**********************************************************************
	pbg̎擾
**********************************************************************/
void CAliceCG_PMS::ExpandPal( ALSD_ALCG::stALCG_Palette *_lpPal )
{
	if( m_Header.Bf & 0x0001 ){
		for( long j = 10; j < 16; j++){
			_lpPal->entry[j] = m_Palette.entry[j];	
		}
	}
	for( long i = 1; i < 15; i++){
		if( m_Header.Bf & ( 1 << i ) ){
			for( long j = i * 16; j < i * 16 + 16; j++){
				_lpPal->entry[j] = m_Palette.entry[j];	
			}
		}
	}
	if( m_Header.Bf & 0x8000 ){
		for( long j = 240; j < 255; j++){
			_lpPal->entry[j] = m_Palette.entry[j];	
		}
	}
}

/**********************************************************************
	256F
**********************************************************************/
void CAliceCG_PMS::Expand256( unsigned char *_lpPic, long _Ptr)
{
	int c0, c1;
	int x, y, loc, l, i;
	int lpitch = m_Header.Width;
	const unsigned char *b = m_lpMem;
	unsigned char *pic = _lpPic;
	
	if( _Ptr == -1 ){
		b += m_Header.DataPtr;
	}else{
		b += _Ptr;
	}

	for( y = 0; y < m_Header.Height; y++){
		for( x = 0; x < m_Header.Width; ){
			loc = y * lpitch + x;
			c0 = *b++;
			if( c0 <= 0xf7 ){
				*(pic + loc) = c0;
				x++;
			}else if( c0 == 0xff ){
				l = (*b) + 3; b++;
				x += l; 
				memcpy( pic + loc, pic + loc - lpitch, l);
			}else if( c0 == 0xfe ){
				l = (*b) + 3; b++;
				x += l; 
				memcpy( pic + loc, pic + loc - lpitch * 2, l);
			}else if( c0 == 0xfd ){
				l = (*b) + 4; b++;
				x += l;
				c0 = *b++;
				memset( pic + loc, c0, l);
			}else if( c0 == 0xfc ){
				l = ( (*b) + 3 )  * 2; b++;
				x += l;
				c0 = *b++;
				c1 = *b++;
				for( i = 0; i < l; i+=2){
					*(pic + loc + i    ) = c0;
					*(pic + loc + i + 1) = c1;
				}
			}else{
				*(pic + loc) = *b++;
				x++;
			}
		}
	}
}
